"use strict";

exports.__esModule = true;
exports.rule = rule;

var _types = require("@babel/types");

var REACT_COMPONENT_EXPORT_NAME = 'Component';
var REACT_COMPONENT_RENDER_FUNCTION_NAME = 'render';
var REACT_MODULE_NAME = 'react';
var REACT_PURE_COMPONENT_EXPORT_NAME = 'PureComponent'; // eslint-disable-next-line flowtype/no-weak-types

function rule(context) {
  var classStack = [];
  var functionStack = [];
  var componentImportedAs;
  var pureComponentImportedAs;
  var reactImportedAs;
  return {
    ArrowFunctionExpression: function ArrowFunctionExpression() {
      functionStack.unshift(false);
    },
    'ArrowFunctionExpression:exit': function ArrowFunctionExpressionExit() {
      functionStack.shift();
    },
    // eslint-disable-next-line flowtype/no-weak-types
    ImportDeclaration: function ImportDeclaration(node) {
      var source = node.source,
          specifiers = node.specifiers;

      if (source.value === REACT_MODULE_NAME && Array.isArray(specifiers)) {
        for (var i = 0; i < specifiers.length; i++) {
          var specifier = specifiers[i];

          if ((0, _types.isImportDefaultSpecifier)(specifier) || (0, _types.isImportNamespaceSpecifier)(specifier)) {
            reactImportedAs = specifier.local.name;
          } else if ((0, _types.isImportSpecifier)(specifier)) {
            switch (specifier.imported.name) {
              case REACT_COMPONENT_EXPORT_NAME:
                componentImportedAs = specifier.local.name;
                break;

              case REACT_PURE_COMPONENT_EXPORT_NAME:
                pureComponentImportedAs = specifier.local.name;
                break;
            }
          }
        }
      }
    },
    // eslint-disable-next-line flowtype/no-weak-types
    ClassDeclaration: function ClassDeclaration(node) {
      var superClass = node.superClass;

      if (superClass) {
        if ((0, _types.isIdentifier)(superClass)) {
          if (componentImportedAs && superClass.name === componentImportedAs || pureComponentImportedAs && superClass.name === pureComponentImportedAs) {
            classStack.unshift(true);
            return;
          }
        } else if ((0, _types.isMemberExpression)(superClass)) {
          if (reactImportedAs && superClass.object.name === reactImportedAs && [REACT_COMPONENT_EXPORT_NAME, REACT_PURE_COMPONENT_EXPORT_NAME].includes(superClass.property.name)) {
            classStack.unshift(true);
            return;
          }
        }
      }

      classStack.push(false);
    },
    // eslint-disable-next-line flowtype/no-weak-types
    'ClassDeclaration:exit': function ClassDeclarationExit(node) {
      classStack.shift();
    },
    FunctionDeclaration: function FunctionDeclaration() {
      functionStack.unshift(false);
    },
    'FunctionDeclaration:exit': function FunctionDeclarationExit() {
      functionStack.shift();
    },
    // eslint-disable-next-line flowtype/no-weak-types
    MethodDefinition: function MethodDefinition(node) {
      functionStack.unshift(node.key.name === REACT_COMPONENT_RENDER_FUNCTION_NAME);
    },
    'MethodDefinition:exit': function MethodDefinitionExit() {
      functionStack.shift();
    },
    // eslint-disable-next-line flowtype/no-weak-types
    ReturnStatement: function ReturnStatement(node) {
      if (classStack[0] && functionStack[0] || reactImportedAs && classStack.length === 0 && functionStack.length === 1) {
        var argument = node.argument;

        if ((0, _types.isArrayExpression)(argument) && argument.elements.some(function (element) {
          return (0, _types.isJSXElement)(element);
        })) {
          context.report({
            message: "Don't return an array fron a React component's render method.",
            node: node.argument
          });
        }
      }
    }
  };
}